import sys, os
from math import *
import numpy, tables, scipy
import Rappture

import warnings
warnings.simplefilter('ignore',UserWarning)

io = Rappture.library(sys.argv[1])
toolDir = os.path.dirname(os.path.abspath(sys.argv[0]))
if toolDir == None:
  toolDir = os.getcwd()

execfile(os.path.join(toolDir,"dot.py"))
execfile(os.path.join(toolDir,"atom.py"))
execfile(os.path.join(toolDir,"wire.py"))
execfile(os.path.join(toolDir,"trap.py"))
execfile(os.path.join(toolDir,"expert.py"))

modeObj = None
mode = io.get('input.phase(one).choice(mode).current')
if mode=="dot":
  modeObj = Dot(io, toolDir)
if mode=="atom":
  modeObj = Atom(io, toolDir)
if mode=="wire":
  modeObj = Wire(io, toolDir)
if mode=="trap":
  modeObj = Trap(io, toolDir)
if mode=="expert":
  modeObj = Expert(io, toolDir)

if modeObj != None:
  modeObj.writeXML()

#Get the physical dimensionality.
ndim = io.get('input.phase(three).choice(ndim).current')

#Option to streamOutput.
streamOutput = (io.get('input.phase(three).boolean(streamOutput).current')
                =="yes")

#Clear old results
for file in ["paths.out","pimc.dat","pi.log","pimc.h5","density.h5"]:
  try:
    os.remove(file)
  except OSError, detail:
    if detail.errno != 2:
      raise

#Run the program 
errStatus,piLOG,piError \
  = Rappture.tools.getCommandOutput('pi%s' % ndim, streamOutput)

#Put the log file.
io.put('output.log',piLOG)

#Analyze the data
if os.path.isfile("pimc.dat"):
#  data = pylab.load("pimc.dat")
   data = numpy.loadtxt("pimc.dat")
   if (numpy.size(data.shape)==1):
     data = data.reshape(data.size,1)

   npts = data.shape[0]
   ndata = data.shape[1]

# Process the pimc.h5 file.
if os.path.isfile("pimc.h5"):

   h5file=tables.openFile("pimc.h5","r")
   npart=h5file.getNode("/simInfo","npart").read()[0]
   nslice=h5file.getNode("/simInfo","nslice").read()[0]
   temperature=h5file.getNode("/simInfo","temperature").read()[0]
   idata = 0
   for node in h5file.walkNodes('/estimators','Array'):
     if (node.name in set(['thermo_energy','virial_energy','coulomb+_energy',
         'coulomb_energy','angular_momentum'])):
       io.put('output.curve(%i).about.label'%idata, node.name)
       io.put('output.curve(%i).about.label'%idata, 
              'Scalar estimator: %s'%node.name)
       io.put('output.curve(%i).xaxis.label'%idata,"Block Number")
       io.put('output.curve(%i).yaxis.label'%idata,"Trace")
       if ("unit" in node.attrs):
         io.put('output.curve(%i).yaxis.units'%idata,node.attrs.unit[0])
       data = numpy.array(node.read())
       av = numpy.average(data)
       for i in range(data.size):
         x = i
         y = data[i]
         io.put('output.curve(%i).component.xy'%idata, '%g %g\n' % (x,y),
	        append=1)
         io.put('output.curve(%i).component(av).xy'%idata, '%g %g\n' % (x,av),
                append=1)
       idata += 1
   for node in h5file.walkNodes('/estimators','Array'):
     if (node.name == 'conductance'):
       io.put('output.curve(%i).about.label'%idata, node.name)
       io.put('output.curve(%i).about.label'%idata, 
              'Imag. Time Corr.: chi_jj')
       io.put('output.curve(%i).xaxis.label'%idata,"iwn/iw0")
       io.put('output.curve(%i).yaxis.label'%idata,"-chi_jj/iwn^2")
       data = numpy.array(node.read())
       nfreq = data.shape[-1]
       omega1 = 2*numpy.pi*temperature
       omegan = omega1*numpy.array(range(1,nfreq));
       alphaiwn = -data[:,:,:,:,1:nfreq+1]/omegan**2;
       if (nfreq>nslice/2):
          nfreq=nslice/2
       for i in range(1,nfreq-1):
         x = i
         y = alphaiwn[0,0,0,0,i]
         io.put('output.curve(%i).component.xy'%idata, '%g %g\n' % (x,y),
                append=1)
       idata += 1

   h5file.close()
   io.put('output.string(pimc_h5).about.label',"pimc.h5")
   io.put('output.string(pimc_h5).current',"pimc.h5",type="file",
          compress="yes")

if os.path.isfile("density.h5"):
   io.put('output.string(density_h5).about.label',"density.h5")
   io.put('output.string(density_h5).current',"density.h5",type="file",
          compress="yes")

if os.path.isfile("paths.out"):
   io.put('output.string(paths_out).about.label',"paths.out")
   io.put('output.string(paths_out).current',"paths.out",type="file",
          compress="yes")

if os.path.isfile("pimc.xml"):
   io.put('output.string(pimc_xml).about.label',"pimc.xml")
   io.put('output.string(pimc_xml).current',"pimc.xml",type="file",
          compress="yes")

Rappture.result(io)
