//$Id: H5ReportBuilder.cc,v 1.12 2007/10/26 07:10:05 jshumwa Exp $
/*  Copyright (C) 2004-2006 John B. Shumway, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "H5ReportBuilder.h"
#include "ScalarEstimator.h"
#include "EstimatorManager.h"
#include "ArrayBlockedEstimator.h"

H5ReportBuilder::H5ReportBuilder(const std::string& filename,
  const EstimatorManager::SimInfoWriter *simInfoWriter) 
  : filename(filename), simInfoWriter(simInfoWriter),
    file(filename,H5F_ACC_TRUNC), writingGroup(0), stepAttr(0), dataset(0) {
  simInfoWriter->writeH5(file);
}

H5ReportBuilder::~H5ReportBuilder() {}

void H5ReportBuilder::startWritingGroup(EstimatorManager& manager) {
  nstep=manager.nstep;
  istep=0;
  writingGroup = new H5::Group(file.createGroup("estimators"));
  hsize_t dims[] = {1};
  stepAttr = new H5::Attribute(writingGroup->createAttribute("nstep",
                       H5::PredType::NATIVE_INT, H5::DataSpace(1,dims)));
  stepAttr->write(H5::PredType::NATIVE_INT, &istep);
  dataset.resize(0);
  for (EstimatorManager::EstimatorIter est=manager.estimator.begin();
       est!=manager.estimator.end(); ++est) {
    (*est)->startReport(*this);
  }
}

void H5ReportBuilder::startScalarReport(const ScalarEstimator& est) {
  hsize_t dims[1]; dims[0]=nstep;
  H5::DataSet* dataSet
    =new H5::DataSet(writingGroup-> createDataSet(est.getName(),
                     H5::PredType::NATIVE_FLOAT,H5::DataSpace(1,dims)));
  const std::string& unitName(est.getUnitName());
  if (unitName!="") {
    dims[0]=1;
    hsize_t length=2;
    H5::StrType strType(H5::PredType::C_S1,length);
    H5::Attribute* attr = new H5::Attribute(dataSet->createAttribute("unit",
           strType, H5::DataSpace(1,dims)));
    attr->write(strType, unitName.c_str());
  }
  dataset.push_back(dataSet);
}

void H5ReportBuilder::startAccRejReport(const AccRejEstimator& est) {

}

void H5ReportBuilder::reportAccRejStep(const AccRejEstimator& est) {
}

void H5ReportBuilder::writeStep(EstimatorManager& manager) {
  dset=dataset.begin();
  for (EstimatorManager::EstimatorIter est=manager.estimator.begin();
       est!=manager.estimator.end(); ++est) {
    (*est)->reportStep(*this);
  }
  stepAttr->write(H5::PredType::NATIVE_INT, &(++istep));
  stepAttr->flush(H5F_SCOPE_LOCAL);
  if (istep == nstep) {
    delete stepAttr; 
    delete writingGroup; 
    for (DataSetIter d=dataset.begin(); d!=dataset.end(); ++d) delete *d; 
    dataset.resize(0);
  }
}

void H5ReportBuilder::reportScalarStep(const ScalarEstimator& est) {
  H5::DataSpace space = (*dset)->getSpace();
  hsize_t element[][1] = {{istep}};
  space.selectNone();
  space.selectElements(H5S_SELECT_SET,1,(const hsize_t **)element);
  float a=est.getValue();
  hsize_t dims[]={1}, maxdims[]={1}; H5::DataSpace mspace(1,dims,maxdims);
  (*dset)->write(&a,H5::PredType::NATIVE_FLOAT,mspace,space);
  dset++;
}
 
void H5ReportBuilder::reportArrayBlockedStep(const ArrayBlockedEstimator& est) {
//  H5::DataSpace space = (*dset)->getSpace();
//  hsize_t element[][1] = {{istep}};
//  space.selectNone();
//  space.selectElements(H5S_SELECT_SET,1,(const hsize_t **)element);
//  float a=est.getValue();
//  hsize_t dims[]={1}, maxdims[]={1}; H5::DataSpace mspace(1,dims,maxdims);
  est.normalize();
  (*dset)->write(est.getData(),H5::PredType::NATIVE_FLOAT);
  dset++;
  if (est.hasError()) {
    (*dset)->write(est.getError(),H5::PredType::NATIVE_FLOAT);
    dset++;
  }
  est.unnormalize();
}

void H5ReportBuilder::startArrayBlockedReport(const ArrayBlockedEstimator& est) {
  hsize_t dims[est.getNDim()];
  for (int i=0; i<est.getNDim(); ++i) dims[i]=est.getExtent(i);
  dataset.push_back(
    new H5::DataSet(writingGroup-> createDataSet(est.getName(),
                    H5::PredType::NATIVE_FLOAT,
                    H5::DataSpace(est.getNDim(),dims))));
  if (est.hasError()) {
    dataset.push_back(
      new H5::DataSet(writingGroup-> createDataSet(est.getName()+"_err",
                      H5::PredType::NATIVE_FLOAT,
                      H5::DataSpace(est.getNDim(),dims))));
  }
}
