// $Id: PairGridAction.h,v 1.3 2006/10/18 17:08:19 jshumwa Exp $
/*  Copyright (C) 2004-2006 John B. Shumway, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
#ifndef __PairGridAction_h_
#define __PairGridAction_h_
class MultiLevelSampler;
class Paths;
class Species;
class SimulationInfo;
#include "Action.h"
#include <blitz/array.h>
#include <vector>
#include <string>

/** Class for calculating pair action between particles.
* We take the interaction from Ken's squarer++ code.
* dependence stored as a power series. We take advantage of the extra
* symmetry of the coulomb action. 
* @version $Revision: 1.3 $
* bug; hard coded for NDIM=2.
* @todo Make a version for non-coulomb actions.
* @author Daejin Shin. */
class PairGridAction : public Action {
public:
  /// Typedefs.
  typedef blitz::Array<int,1> IArray;
  typedef blitz::Array<double,1> Array;
  typedef blitz::Array<double,2> Array2;
  /// Constructor by providing the species and dmu filename.
  PairGridAction(const Species&, const Species&, const std::string& filename,
             const SimulationInfo& );
  /// Virtual destructor.
  virtual ~PairGridAction() {}
  /// Calculate the difference in action.
  virtual double getActionDifference(const MultiLevelSampler&,
                                     const int level);
  /// Calculate the total action.
  virtual double getTotalAction(const Paths&, const int level) const;
  /// Calculate the action and derivatives at a bead.
  virtual void getBeadAction(const Paths&, const int ipart, const int islice,
    double& u, double& utau, double& ulambda, Vec& fm, Vec& fp) const;
  friend class EwaldAction;
private:
  /// The timestep.
  const double tau;
  /// The grid dimensions
  int n1, n2;
  /// Inverse grid spacings.
  double delqinv, deltinv;
  /// The grid spacings.
  double delq, delt;
  /// The number of gridpoints.
  int ngpts;
  /// The action grid.
  std::vector<Array2> ugrid;
  /// Evalute the action from the grid.
  double v(double q, double t) const;
 /// Evalute the action from the grid.
  double dv(double q, double t) const;
  /// Evalute the diagonal action from the grid.
  int ifirst1,ifirst2;
  /// Number of each particle for species in the interaction.
  int npart1,npart2;
};
#endif
