// $Id: WriteProbDensity.cc,v 1.7 2006/10/18 17:08:19 jshumwa Exp $
/*  Copyright (C) 2004-2006 John B. Shumway, Jr.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef ENABLE_MPI
#include <mpi.h>
#endif
#include "WriteProbDensity.h"
#include "ProbDensityGrid.h"
#include "SimulationInfo.h"
#include <H5Cpp.h>
#include <blitz/array.h>

WriteProbDensity::WriteProbDensity(const SimulationInfo& simInfo,
  const ProbDensityGrid* grid, const std::string& filename)
  : filename(filename), grids(grid), name(simInfo.getNSpecies(),"rho") {
  for (unsigned int i=0; i<name.size(); ++i) {
    name[i]+=simInfo.getSpecies(i).name;
  }
}

WriteProbDensity::~WriteProbDensity() {}

void WriteProbDensity::run() {
  int rank=0;
#ifdef ENABLE_MPI
  rank=MPI::COMM_WORLD.Get_rank();
#endif
  //Output the density to output file.
  H5::H5File* file(0);
  if (rank==0) file=new H5::H5File(filename,H5F_ACC_TRUNC);
  IVec n=grids->getGridDim();
  const double norm=grids->getNorm(); double normsum=norm;
#ifdef ENABLE_MPI
  MPI::COMM_WORLD.Reduce(&norm,&normsum,1,MPI::DOUBLE,MPI::SUM,0);
#endif
  const double anorm=1.0/normsum;
  FArrayN floatgrid(n),sumgrid(n);
  for (unsigned int i=0; i<name.size(); ++i) {
    const LIArrayN& intgrid(grids->getGrid(i));
    floatgrid=1.0*intgrid;
    sumgrid=floatgrid;
#ifdef ENABLE_MPI
    MPI::COMM_WORLD.Reduce(floatgrid.data(),sumgrid.data(),product(n),
                           MPI::FLOAT,MPI::SUM,0);
#endif
    sumgrid*=anorm;
    if (rank==0) {
      hsize_t dims[NDIM];
      for (int idim=0; idim<NDIM; ++idim) dims[idim]=n[idim];
      H5::DataSpace dataSpace(NDIM,dims);
      H5::DataSet rho(file->createDataSet(name[i], H5::PredType::NATIVE_FLOAT,
                   dataSpace));
      rho.write(sumgrid.data(),H5::PredType::NATIVE_FLOAT);
      {/// Write a few more attributes for use with DX ImportHDF5Field
        double a=grids->getLatticeConstant(); hsize_t dims[]={NDIM};
        Vec origin;
        for (int idim=0; idim<NDIM; ++idim) origin[idim]=-0.5*a*n[idim];
        (rho.createAttribute("origin",
               H5::PredType::NATIVE_FLOAT, H5::DataSpace(1,dims))) 
          .write(H5::PredType::NATIVE_DOUBLE,&origin);
        Vec delta; delta=a;
        (rho.createAttribute("delta",
               H5::PredType::NATIVE_FLOAT, H5::DataSpace(1,dims)))
          .write(H5::PredType::NATIVE_DOUBLE,&delta);
      }
    }
  }
  if (rank==0) {
    double a=grids->getLatticeConstant(); hsize_t dims[]={1};
    H5::DataSet datasetA(file->createDataSet("a",
            H5::PredType::NATIVE_FLOAT, H5::DataSpace(1,dims))); 
    datasetA.write(&a,H5::PredType::NATIVE_DOUBLE);
  }
  delete file;
}
